#!/usr/bin/python
# -*- coding: utf-8 -*-

from appscript import *
from itertools import izip
import htmlentitydefs
import re

# based on <http://sebsauvage.net/python/snyppets/>
RE_ENTITY_CHR = re.compile(u'&(%s);' % u'|'.join(htmlentitydefs.name2codepoint))
RE_ENTITY_DEC = re.compile(u'&#(\d+);')
RE_ENTITY_HEX = re.compile(u'&#x(\w+);')
def decode_entities(s):
    def entity2char(m):
        entity = m.group(1)
        if entity in htmlentitydefs.name2codepoint:
            return unichr(htmlentitydefs.name2codepoint[entity])
        return u' '  # Unknown entity: We replace with a space.
    replaced = 0
    s, n = RE_ENTITY_CHR.subn(entity2char, s)
    replaced += n
    s, n = RE_ENTITY_DEC.subn(lambda x: unichr(int(x.group(1))), s)
    replaced += n
    s, n = RE_ENTITY_HEX.subn(lambda x: unichr(int(x.group(1),16)), s)
    replaced += n
    return s, replaced

OPEN_P = r'<[Pp][^>]*>'
RE_OPEN_P = re.compile(OPEN_P)
RE_MATCHED_P = re.compile(OPEN_P + r'(?!<[Pp])(.*)</[Pp]>')
RE_TOO_MANY_CR = re.compile(r'\s*\n\s*\n\s*\n+', re.U)
RE_TAG = re.compile(r'<[^>]*>')
RE_WHITESPACE = re.compile(r'\s+', re.U)
RE_LEADING_WHITESPACE = re.compile(r'^\s+(.*)', re.U)
RE_TRAILING_WHITESPACE = re.compile(r'(.*)\s+$', re.U)

def html_to_text(s):
    s, replaced = decode_entities(s)
    s = s.replace('\r\n', '\n')
    s = s.replace('\r', '\n')
    if replaced > 0 or RE_TAG.search(s): # HTML
        s = RE_WHITESPACE.sub(' ', s)
        s = RE_MATCHED_P.sub(r'\1\n\n', s)
        s = RE_OPEN_P.sub(r'\n\n', s)
        s = RE_TAG.sub('', s)
        s = RE_TOO_MANY_CR.sub(r'\n\n', s)
    s = RE_LEADING_WHITESPACE.sub(r'\1', s)
    s = RE_TRAILING_WHITESPACE.sub(r'\1', s)
    return s

def podcasts_to_lyrics():
    iTunes = app('iTunes')

    podcasts = iTunes.tracks[its.podcast == True]

    ids = podcasts.id()
    descs = podcasts.description()
    longdescs = podcasts.long_description()
    lyricses = podcasts.lyrics()

    for id_, desc, longdesc, lyrics in izip(ids, descs, longdescs, lyricses):
        if lyrics == k.missing_value: # video
            continue

        if longdesc == k.missing_value:
            if desc == k.missing_value:
                continue
            longdesc = desc
        else:
            longdesc = html_to_text(longdesc)

        if lyrics == longdesc:
            continue

        iTunes.tracks[its.id == id_].lyrics.set(longdesc)



if __name__ == '__main__':
    # sync_ipod()
    # - poll whether 'Sync' button is enabled
    # update_podcasts()
    # - poll '[queued for download]' on last podcast, then any remaining
    podcasts_to_lyrics()
    # sync_ipod()

# write podcast entry replacing http://njr.sabi.net/2007/03/04/an-applescript-to-update-podcasts-and-your-ipod/ - now we can use accessibility
